@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,11 +42,6 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
-#  include <wchar.h>
-#  include <wctype.h>
-#endif
-
 #include "gtkutils.h"
 #include "utils.h"
 #include "codeconv.h"
@@ -420,6 +415,31 @@ gboolean gtkut_tree_model_find_by_column_data(GtkTreeModel *model,
 	return FALSE;
 }
 
+void gtkut_tree_model_foreach(GtkTreeModel *model, GtkTreeIter *start,
+			      GtkTreeModelForeachFunc func, gpointer user_data)
+{
+	gboolean valid = TRUE;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+
+	g_return_if_fail(func != NULL);
+
+	if (!start) {
+		gtk_tree_model_foreach(model, func, user_data);
+		return;
+	}
+
+	path = gtk_tree_model_get_path(model, start);
+	func(model, path, start, user_data);
+	gtk_tree_path_free(path);
+
+	valid = gtk_tree_model_iter_children(model, &iter, start);
+	while (valid) {
+		gtkut_tree_model_foreach(model, &iter, func, user_data);
+		valid = gtk_tree_model_iter_next(model, &iter);
+	}
+}
+
 gboolean gtkut_tree_row_reference_get_iter(GtkTreeModel *model,
 					   GtkTreeRowReference *ref,
 					   GtkTreeIter *iter)
@@ -560,7 +580,8 @@ void gtkut_tree_view_vertical_autoscroll(GtkTreeView *treeview)
 }
 
 /* modified version of gtk_tree_view_scroll_to_cell */
-void gtkut_tree_view_scroll_to_cell(GtkTreeView *treeview, GtkTreePath *path)
+void gtkut_tree_view_scroll_to_cell(GtkTreeView *treeview, GtkTreePath *path,
+				    gboolean align_center)
 {
 	GdkRectangle cell_rect;
 	GdkRectangle vis_rect;
@@ -580,18 +601,41 @@ void gtkut_tree_view_scroll_to_cell(GtkTreeView *treeview, GtkTreePath *path)
 
 	/* add margin */
 	if (cell_rect.height * 2 < vis_rect.height)
-		margin = cell_rect.height + 2;
+		margin = cell_rect.height + (align_center ? 0 : 2);
 
-	if (cell_rect.y < vis_rect.y + margin)
-		dest_y = cell_rect.y - margin;
+	if (cell_rect.y < vis_rect.y + margin) {
+		if (align_center)
+			dest_y = cell_rect.y -
+				(vis_rect.height - cell_rect.height) / 2;
+		else
+			dest_y = cell_rect.y - margin;
+	}
 	if (cell_rect.y + cell_rect.height >
-		vis_rect.y + vis_rect.height - margin)
-		dest_y = cell_rect.y + cell_rect.height - vis_rect.height +
-			margin;
+		vis_rect.y + vis_rect.height - margin) {
+		if (align_center)
+			dest_y = cell_rect.y -
+				(vis_rect.height - cell_rect.height) / 2;
+		else
+			dest_y = cell_rect.y + cell_rect.height -
+				vis_rect.height + margin;
+	}
 
 	gtk_tree_view_scroll_to_point(treeview, dest_x, dest_y);
 }
 
+void gtkut_tree_view_fast_clear(GtkTreeView *treeview, GtkTreeStore *store)
+{
+#if GTK_CHECK_VERSION(2, 8, 0) && !GTK_CHECK_VERSION(2, 10, 0)
+	gtk_tree_store_clear(store);
+#else
+	/* this is faster than above, but it seems to trigger crashes in
+	   GTK+ 2.8.x */
+	gtk_tree_view_set_model(treeview, NULL);
+	gtk_tree_store_clear(store);
+	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
+#endif
+}
+
 void gtkut_combo_set_items(GtkCombo *combo, const gchar *str1, ...)
 {
 	va_list args;
@@ -616,7 +660,7 @@ void gtkut_combo_set_items(GtkCombo *combo, const gchar *str1, ...)
 
 gchar *gtkut_editable_get_selection(GtkEditable *editable)
 {
-	guint start_pos, end_pos;
+	gint start_pos, end_pos;
 	gboolean found;
 
 	g_return_val_if_fail(GTK_IS_EDITABLE(editable), NULL);
@@ -837,6 +881,26 @@ void gtkut_window_popup(GtkWidget *window)
 	gtk_window_present(GTK_WINDOW(window));
 }
 
+gboolean gtkut_window_modal_exist(void)
+{
+	GList *window_list, *cur;
+	gboolean exist = FALSE;
+
+	window_list = gtk_window_list_toplevels();
+	for (cur = window_list; cur != NULL; cur = cur->next) {
+		GtkWidget *window = GTK_WIDGET(cur->data);
+
+		if (GTK_WIDGET_VISIBLE(window) &&
+		    gtk_window_get_modal(GTK_WINDOW(window))) {
+			exist = TRUE;
+			break;
+		}
+	}
+	g_list_free(window_list);
+
+	return exist;
+}
+
 void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
 {
 	gint x, y;
