@@ -367,6 +367,7 @@ static void imp_ldif_finish_show() {
 		sMsg = mgu_error2string( _ldifFile_->retVal );
 	}
 	imp_ldif_status_show( sMsg );
+	gtk_button_set_label(GTK_BUTTON(impldif_dlg.btnCancel), GTK_STOCK_CLOSE);
 	gtk_widget_grab_focus(impldif_dlg.btnCancel);
 }
 
@@ -759,9 +760,9 @@ static void imp_ldif_dialog_create() {
 
 	/* Status line */
 	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
+	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, 0);
 	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	/* Button panel */
 	gtkut_stock_button_set_create(&hbbox, &btnPrev, _("Prev"),
@@ -819,6 +820,7 @@ AddressBookFile *addressbook_imp_ldif( AddressIndex *addrIndex ) {
 	gtk_notebook_set_current_page( GTK_NOTEBOOK(impldif_dlg.notebook), PAGE_FILE_INFO );
 	gtk_widget_set_sensitive( impldif_dlg.btnPrev, FALSE );
 	gtk_widget_set_sensitive( impldif_dlg.btnNext, TRUE );
+	gtk_button_set_label( GTK_BUTTON(impldif_dlg.btnCancel), GTK_STOCK_CANCEL );
 	stock_pixmap_gdk( impldif_dlg.window, STOCK_PIXMAP_MARK,
 			  &markxpm, &markxpmmask );
 	imp_ldif_message();
