@@ -42,10 +42,12 @@
 #  include <libpisock/pi-args.h>
 #  include <libpisock/pi-appinfo.h>
 #  include <libpisock/pi-address.h>
+#  include <libpisock/pi-version.h>
 #else
 #  include <pi-args.h>
 #  include <pi-appinfo.h>
 #  include <pi-address.h>
+#  include <pi-version.h>
 #endif
 
 #include "mgutils.h"
@@ -339,7 +341,7 @@ static gboolean jpilot_mark_files( JPilotFile *pilotFile ) {
 	pilotFile->pc3ModifyTime = 0;
 	pcFile = jpilot_get_pc3_file( pilotFile );
 	if( pcFile == NULL ) return retVal;
-	if( 0 == lstat( pcFile, &filestat ) ) {
+	if( 0 == g_lstat( pcFile, &filestat ) ) {
 		pilotFile->havePC3 = TRUE;
 		pilotFile->pc3ModifyTime = filestat.st_mtime;
 		retVal = TRUE;
@@ -367,7 +369,7 @@ static gboolean jpilot_check_files( JPilotFile *pilotFile ) {
 	pcFile = jpilot_get_pc3_file( pilotFile );
 	if( pcFile == NULL ) return FALSE;
 
-	if( 0 == lstat( pcFile, &filestat ) ) {
+	if( 0 == g_lstat( pcFile, &filestat ) ) {
 		if( filestat.st_mtime == pilotFile->pc3ModifyTime ) retVal = FALSE;
 	}
 	g_free( pcFile );
@@ -644,7 +646,7 @@ static gint jpilot_get_file_info( JPilotFile *pilotFile, unsigned char **buf, in
 	*buf_size=0;
 
 	if( pilotFile->path ) {
-		in = fopen( pilotFile->path, "rb" );
+		in = g_fopen( pilotFile->path, "rb" );
 		if( !in ) {
 			return MGU_OPEN_FILE;
 		}
@@ -826,7 +828,7 @@ static gint jpilot_read_db_files( JPilotFile *pilotFile, GList **records ) {
 		return MGU_BAD_ARGS;
 	}
 
-	in = fopen( pilotFile->path, "rb" );
+	in = g_fopen( pilotFile->path, "rb" );
 	if (!in) {
 		return MGU_OPEN_FILE;
 	}
@@ -943,7 +945,7 @@ static gint jpilot_read_db_files( JPilotFile *pilotFile, GList **records ) {
 	/* Read the PC3 file, if present */
 	pcFile = jpilot_get_pc3_file( pilotFile );
 	if( pcFile == NULL ) return MGU_SUCCESS;
-	pc_in = fopen( pcFile, "rb");
+	pc_in = g_fopen( pcFile, "rb");
 	g_free( pcFile );
 
 	if( pc_in == NULL ) {
@@ -1009,10 +1011,22 @@ static void jpilot_load_address( JPilotFile *pilotFile, buf_rec *buf, ItemFolder
 	struct AddressAppInfo *ai;
 	gchar **firstName = NULL;
 	gchar **lastName = NULL;
+#if (PILOT_LINK_MAJOR > 11)
+	pi_buffer_t *RecordBuffer;
+#endif /* PILOT_LINK_0_12 */
 
 	/* Retrieve address */
+#if (PILOT_LINK_MAJOR < 12)
 	num = unpack_Address( & addr, buf->buf, buf->size );
 	if( num > 0 ) {
+#else /* PILOT_LINK_0_12 */
+	RecordBuffer = pi_buffer_new(buf->size);
+	memcpy(RecordBuffer->data, buf->buf, buf->size);
+	RecordBuffer->used = buf->size;
+	num = unpack_Address( & addr, RecordBuffer, address_v1 );
+	pi_buffer_free(RecordBuffer);
+	if (num != -1) {
+#endif
 		addrEnt = addr.entry;
 		attrib = buf->attrib;
 		unique_id = buf->unique_id;
@@ -1641,7 +1655,7 @@ gchar *jpilot_find_pilotdb( void ) {
 	strcat( str, JPILOT_DBHOME_FILE );
 
 	/* Attempt to open */
-	if( ( fp = fopen( str, "rb" ) ) != NULL ) {
+	if( ( fp = g_fopen( str, "rb" ) ) != NULL ) {
 		fclose( fp );
 	}
 	else {
