@@ -142,12 +142,13 @@ static void sig_status_for_key(GString *str, gpgme_ctx_t ctx,
 			g_string_sprintfa
 				(str, "Key fingerprint: %s\n", sig->fpr);
 		g_string_append(str, _("Cannot find user ID for this key."));
+		g_string_append(str, "\n");
 		return;
 	}
 	user = key->uids;
 	g_string_sprintfa
 		(str, gpgmegtk_sig_status_to_string (sig, TRUE), user->uid);
-	g_string_append (str, "\n");
+	g_string_append(str, "\n");
 
 	user = user->next;
 	while (user) {
@@ -361,12 +362,14 @@ static gchar *copy_gpgmedata_to_temp(GpgmeData data, guint *length)
 }
 #endif
 
-static gpgme_data_t pgp_decrypt(MimeInfo *partinfo, FILE *fp)
+static gpgme_data_t pgp_decrypt(MsgInfo *msginfo, MimeInfo *partinfo, FILE *fp)
 {
 	gpgme_ctx_t ctx = NULL;
 	gpgme_error_t err;
 	gpgme_data_t cipher = NULL, plain = NULL;
 	struct passphrase_cb_info_s info;
+	gpgme_verify_result_t verifyresult = NULL;
+	const gchar *result = NULL;
 
 	memset(&info, 0, sizeof info);
 
@@ -395,18 +398,42 @@ static gpgme_data_t pgp_decrypt(MimeInfo *partinfo, FILE *fp)
 		gpgme_set_passphrase_cb(ctx, gpgmegtk_passphrase_cb, &info);
 	}
 
-	err = gpgme_op_decrypt(ctx, cipher, plain);
+	err = gpgme_op_decrypt_verify(ctx, cipher, plain);
+
+	msginfo->encinfo = g_new0(MsgEncryptInfo, 1);
 
-leave:
-	gpgme_data_release(cipher);
 	if (err) {
 		gpgmegtk_free_passphrase();
 		debug_print("decryption failed: %s\n", gpgme_strerror(err));
 		gpgme_data_release(plain);
 		plain = NULL;
-	} else
-		debug_print("** decryption succeeded\n");
+		msginfo->encinfo->decryption_failed = TRUE;
+		goto leave;
+	}
 
+	debug_print("** decryption succeeded\n");
+
+	verifyresult = gpgme_op_verify_result(ctx);
+	if (verifyresult && verifyresult->signatures) {
+		result = gpgmegtk_sig_status_to_string(verifyresult->signatures,
+						       FALSE);
+		msginfo->encinfo->sigstatus = g_strdup(result);
+		msginfo->encinfo->sigstatus_full =
+			sig_status_full(ctx, verifyresult);
+		debug_print("verification status: %s\n", result);
+		debug_print("full status: %s\n",
+			    msginfo->encinfo->sigstatus_full);
+		if (prefs_common.gpg_signature_popup) {
+			GpgmegtkSigStatus statuswindow;
+			statuswindow = gpgmegtk_sig_status_create();
+			gpgmegtk_sig_status_update(statuswindow, ctx);
+			gpgmegtk_sig_status_destroy(statuswindow);
+		}
+	}
+
+
+leave:
+	gpgme_data_release(cipher);
 	gpgme_release(ctx);
 	return plain;
 }
@@ -579,7 +606,8 @@ static gint headerp(gchar *p, gchar **names)
 #define DECRYPTION_ABORT() \
 { \
 	procmime_mimeinfo_free_all(tmpinfo); \
-	msginfo->decryption_failed = 1; \
+	if (msginfo->encinfo) \
+		msginfo->encinfo->decryption_failed = TRUE; \
 	return; \
 }
 
@@ -635,7 +663,7 @@ void rfc2015_decrypt_message(MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
 
 	debug_print("** yep, it is pgp encrypted\n");
 
-	plain = pgp_decrypt(partinfo, fp);
+	plain = pgp_decrypt(msginfo, partinfo, fp);
 	if (!plain) {
 		DECRYPTION_ABORT();
 	}
@@ -688,8 +716,7 @@ void rfc2015_decrypt_message(MsgInfo *msginfo, MimeInfo *mimeinfo, FILE *fp)
 	fclose(dstfp);
 	procmime_mimeinfo_free_all(tmpinfo);
 
-	msginfo->plaintext_file = fname;
-	msginfo->decryption_failed = 0;
+	msginfo->encinfo->plaintext_file = fname;
 }
 
 #undef DECRYPTION_ABORT
@@ -718,14 +745,17 @@ FILE *rfc2015_open_message_decrypted(MsgInfo *msginfo, MimeInfo **mimeinfo)
 	}
 
 	if (MSG_IS_ENCRYPTED(msginfo->flags) &&
-	    !msginfo->plaintext_file &&
-	    !msginfo->decryption_failed) {
+	    (!msginfo->encinfo ||
+	     (!msginfo->encinfo->plaintext_file &&
+	      !msginfo->encinfo->decryption_failed))) {
 		fpos = ftell(fp);
 		rfc2015_decrypt_message(msginfo, mimeinfo_, fp);
-		if (msginfo->plaintext_file &&
-		    !msginfo->decryption_failed) {
+		if (msginfo->encinfo &&
+		    msginfo->encinfo->plaintext_file &&
+		    !msginfo->encinfo->decryption_failed) {
 			fclose(fp);
 			procmime_mimeinfo_free_all(mimeinfo_);
+
 			if ((fp = procmsg_open_message(msginfo)) == NULL)
 				return NULL;
 			mimeinfo_ = procmime_scan_mime_header(fp);
